@@ -158,6 +158,8 @@ HID_REPORT_TYPE_INPUT = 1
 HID_REPORT_TYPE_OUTPUT = 2
 HID_REPORT_TYPE_FEATURE = 3
 
+EVIOCGRAB = _IOW('E', 0x90, _integer)
+
 @_decorator
 def _need_opened(method, self, *args, **kwargs):
 	if self._closed:
@@ -242,8 +244,6 @@ class Unit:
 			return 'H'
 		elif unitval == 0x00e0e121 and unit_exponent == 5:
 			return 'Ω'
-		elif unitval == 0x00e0e121 and unit_exponent == 5:
-			return 'Ω'
 		elif unitval == 0x00f0d121 and unit_exponent == 5:
 			return 'V'
 		elif unitval == 0x01f0 and unit_exponent == 0:
@@ -550,6 +550,7 @@ class HIDDevice:
 		self._interface = None
 		self._busnum = None
 		self._devnum = None
+		self._sys_path = None
 
 		if isinstance(device, str):
 			devicestr = device
@@ -559,6 +560,7 @@ class HIDDevice:
 				raise ValueError('Invalid argument for HIDDevDevice(): {!r}'.format(device))
 		elif hasattr(device, 'sys_path'):
 			self._device_node = device.device_node
+			self._sys_path = device.device_path
 			usbdev = device.find_parent('usb', 'usb_device')
 			usbif = device.find_parent('usb', 'usb_interface')
 			if usbdev:
@@ -571,14 +573,13 @@ class HIDDevice:
 				self._interface = int(usbif.sys_name.rsplit('.',1)[-1])
 		else:
 			raise TypeError('Invalid argument for HIDDevDevice(): {!r}'.format(device))
-
 	
 	def open(self):
 		'''Open the device. This happens implicity on access to most properties and methods.'''
 		if self._closed:
 			raise ValueError('I/O operation on closed device.')
 		if self._file is None:
-			self._file = open(self.device_node, 'wb+', buffering=0)
+			self._file = open(self.device_node, 'rb', buffering=0)
 			self._opened = True
 			# hiddev_usage_ref on read mode
 			self._ioctl_write_struct(HIDIOCSFLAG, _integer(1))
@@ -589,7 +590,7 @@ class HIDDevice:
 		if self._opened:
 			self._opened = False
 			self._file.close()
-
+	
 	def setNonBlocking(self, nonblocking=True):
 		nf = fcntl.fcntl(self.fileno(),fcntl.F_GETFL)
 		if nonblocking:
@@ -688,6 +689,17 @@ class HIDDevice:
 				result.append(0)
 
 		return tuple(result)
+	
+	@property
+	def sys_path(self):
+		if self._sys_path:
+			return self._sys_path
+		else:
+			ctx = pyudev.Context()
+			for item in ctx.list_devices(subsystem='usb', DEVNAME=self.device_node):
+				self._sys_path = item.device_path
+				return self._sys_path
+		return None
 
 	@_need_opened
 	def fileno(self):
@@ -838,13 +850,18 @@ class HIDDevice:
 		r, rstruct = self._ioctl_readwrite_struct(HIDIOCGREPORTINFO, rstruct)
 		return Report(self, rstruct), struct.field_index, struct.usage_index
 
-def enumerate_udev(context=None) -> 'iterator(HIDDevice)':
+def enumerate_udev(context=None, *, vendor_id=None, model_id=None) -> 'iterator(HIDDevice)':
 	'''
 		Enumerate all HID devices via UDev
 	'''
 	if not context:
 		context = pyudev.Context()
-	devices = context.list_devices(subsystem='usb')
+	devices = context.list_devices()
+	devices.match_subsystem('usbmisc')
+	devices.match_subsystem('usb')
 	for device in devices:
 		if device.sys_name.startswith('hiddev'):
-			yield HIDDevice(device)
+			dev = HIDDevice(device)
+			if vendor_id is None or vendor_id == dev.vendor_id:
+				if model_id is None or model_id == dev.model_id:
+					yield dev
